#!/usr/bin/python3
# -*- coding: utf-8 -*-

####################################################################################
# Projet : traduction du pooler Newtest, du langage Perl vers Python,              #
#        ce script utilise les données récupérées sur l'état des robots par        #
#        le pooler Newtest_pooler.py pour les mettre en forme dans un format       #
#        lisible par l'outil de supervision Nagios                                 #
#                                                                                  #
# Auteur : Sherwin SABRAS (https://www.linkedin.com/in/sherwin-sabras-321164172/)  #
# Date de création : 3 juin 2019                                                   #
#                                                                                  #
# Release : 2019-09-24 YSI  v2.0.1                                                 #
####################################################################################

import sys, os
import Newtestlib as Ntl
import re

#########################################################################################
#                                                                                       #
#                               Plugin Robot Version 2 - PYTHON                         #
#                                                                                       #
#########################################################################################


##########################################################################################################
ERRORS = {'OK':0,'WARNING':1,'CRITICAL':2,'UNKNOWN':3,'DEPENDENT':4}

#######################################################################################################################################

#arguments de la ligne de commande
arg = sys.argv

#vérifier les arguments
param_ok = Ntl.more_param(arg)
if param_ok == "no-ok":
    sys.exit()

else:
    #si le chemin d'accès vers le fichier de configuration n'existe pas
    if not os.path.isfile(arg[0]):
        print("config file {} doesn't exist \n".format(arg[0]))
        sys.exit()

path = os.path.basename(__file__)

#on supprime le premier argument entré via la ligne de commande
#(correspondant au chemin d'accès du script exécuté)
if os.path.basename(arg[0]) == path:
    del arg[0]

config_file = arg[0] #le chemin et nom du fichier de configuration newtest_config.cfg
case = arg[1] #mode passif ou actif
nmc = arg[2].replace(".", "_") #l'adresse du nmc, avec les "." remplacés par des "_" 
rbt = arg[3].upper() #nom du robot (en majuscules)

#arg[4] : le nom du scenario pour le cas actif, le nom de l'host pour le cas passif
passif_arg = Ntl.get_passif_arg()

Ntl.get_config_file(arg[0]) #donner le fichier de configuration à la librairie Newtestlib

param = Ntl.get_param() #récupérer les paramètres indiqués dans le fichier de configuration

#récupérer le chemin d'accès complet du fichier trace
suffix_preffix_trace_plug_rbt = Ntl.get_suffix_preffix_trace_plug_rbt()
log = param[2] + suffix_preffix_trace_plug_rbt[0] + nmc + suffix_preffix_trace_plug_rbt[1] #nom du log

#récupérer le chemin d'accès complet des données du pooler
suffix_preffix_file_pooler = Ntl.get_suffix_preffix_file_pooler()
file_pooler = param[2] + suffix_preffix_file_pooler[0] + nmc + suffix_preffix_file_pooler[1] #nom du pooler

#ouverture du fichier log (ajout):
#s'il existe déjà, ouvrir directement
if os.path.isfile(log):
    try:
        LOG = open(log,'a')
    except IOError:
        print(" error when opening the file {}\n".format(log))

#sinon, le créer d'abord
else:
    try:
        LOG = open(log,'a+')
    except IOError:
        print(" error when opening the file {}\n".format(log))

Ntl.write_file(LOG,"******{}:Newtest_plugin_rbt.py*******\n".format(arg[3]))
#print(file_pooler)

#ouverture du pooler (lecture):
try:
    FILE = open(file_pooler,'r')
except IOError:
    print(" error when opening the file {}\n".format(file_pooler))

#taille du fichier généré par le pooler
if not os.path.isfile(file_pooler):
        length_file_pooler = 0
else:
        length_file_pooler = os.path.getsize(file_pooler)
        #print(length_file_pooler)

#tant que ce fichier est vide, fermer puis rouvrir le fichier
while length_file_pooler == 0:
    Ntl.write_file(LOG,"{}:UNKNOWN:file is empty please wait".format(arg[3]))

    try:
        FILE = open(file_pooler,'r')
        length_file_pooler = os.path.getsize(file_pooler)

    except:
        print(" error when opening the file {}\n".format(file_pooler))

#récupérer le mot-clé associé aux robots
word_for_robot = Ntl.get_word_for_robot()

#lire les lignes du pooler
lines = FILE.readlines()

#initialisation des variables
cpt = 0 #numero de la ligne actuelle
robot_found = False #booléen pour indiquer si le robot a été trouvé ou non dans le fichier pooler

#chercher la ligne correspondant au robot indiqué (dans les arguments)
while not robot_found and cpt < len(lines):
    line = lines[cpt]
    words = re.split(r'\|\|',line) #récupérer les informations du pooler, sur chaque ligne, séparées par '||'
    if words[0] == word_for_robot and words[1] == rbt: #si, parmi les lignes RBT, on trouve le robot cherché...
        line_rbt = words.copy() #...sauvegarder la ligne correspondante
        robot_found = True #le robot a été trouvé
    cpt+=1

FILE.close()

#si le robot n'a pas été trouvé
if not robot_found:
    print("error: the name of robot {} doesn't exist".format(rbt))
    Ntl.write_file(LOG,"error: the name of robot {} doesn't exist".format(rbt))
    sys.exit(3)
    
#################APPEL PLUGIN SCE en mode passif
if arg[1] == passif_arg: #mode passif
    #récupérer le chemin d'accès absolu du plugin 'Newtest_plugin_rbt_sce'
    road_sce = Ntl.exec_plug_sce_passif()
    passif_log = "passif.log"

    #exécuter un script Python externe
    os.system("{} {} {} {} {} {} >> {}{}".format(road_sce,arg[0],arg[1],arg[2],arg[3],arg[4],param[3],passif_log))

    Ntl.write_file(LOG,"{}:execution passive sce,with:python {} {} {} {} {} {}\n".format(arg[3],road_sce,arg[0],arg[1],arg[2],arg[3],arg[4]))

##################################
if robot_found:
    #récupérer la date du dernier message (en fin de ligne) et enlever le retour chariot
    date = re.sub('\n+','',line_rbt[3])

    #récupérer l'état du robot
    state_nagios = Ntl.get_nagios_rbt_state(line_rbt[2])
    rbt_state = Ntl.get_rbt_state()

    #distinguer les cas selon l'état du robot
    if line_rbt[2] == rbt_state[0]: #'OK'
        print("{}:probe newtest is running, last message date is at {}|Status={};;;;\n".format(state_nagios[0],date,state_nagios[2]))
        Ntl.write_file(LOG,"{}:{}:probe newtest is running, last message date is at {} |Status={};;;;\n".format(arg[3],state_nagios[0],date,state_nagios[2]))
                    
    elif line_rbt[2] == rbt_state[1]: #'SUSPENDED'
        print("{}:probe newtest is suspended, last message date is at {}|Status={};;;;\n".format(state_nagios[0],date,state_nagios[2]))
        Ntl.write_file(LOG,"{}:{}:probe newtest is suspended, last message date is at {} |Status={};;;;\n".format(arg[3],state_nagios[0],date,state_nagios[2]))

    elif line_rbt[2] in rbt_state[2:7]: #'STOPPED','UNKNOWN','UNINSTALLED','INOPERATIVE','RESUME'
        print("{}:probe newtest is {}, last message date is at {}|Status={};;;;\n".format(state_nagios[0],line_rbt[2],date,state_nagios[2]))
        Ntl.write_file(LOG,"{}:{}:probe newtest is {}, last message date is at {} |Status={};;;;\n".format(arg[3],state_nagios[0],line_rbt[2],date,state_nagios[2]))

    #return nagios exit code
    sys.exit(int(state_nagios[1]))

#fermer le fichier LOG
LOG.close()