#!/usr/bin/python3
# -*- coding: utf-8 -*-

####################################################################################
# Projet : traduction de la librairie Newtestlib, du langage Perl vers Python,     #
#        cette librairie présente un ensemble de fonctions permettant de collecter #
#        des informations sur des machines Newtest                                 #
#                                                                                  #
# Auteur : Sherwin SABRAS (https://www.linkedin.com/in/sherwin-sabras-321164172/)  #
# Date de création : 3 juin 2019                                                   #
#                                                                                  #
# Release : 2019-09-24 YSI  v2.0.1                                                 #
####################################################################################

### Modules ###
# gestion des expressions regulieres (regex)
import re

# fonctions système (exit,...)
import sys
import os

# encodage base64
import base64
# requêtes HTTP
import urllib.request
# erreurs liées aux requêtes
import urllib.error

# parseur xml
# traduire une chaîne XML en dictionnaire (JSON)
import xmltodict

# date et temps
import time

# fonction partie entière (math.floor)
from math import floor

# copie de fichiers
from shutil import copyfile

##########################################################################################################
#                                                                                                        #
#                               Librairie Newtestlib Version 2 - PYTHON                                  #
#                                                                                                        #
##########################################################################################################

##########################################################################################################
### Variables globales ###
debug = 0
word_rbt = "RBT"
word_sce = "SCE"

rbt_sce = [['', '', '', '']]
liste_sce = [['', '', '', '', '']]

##########################################################################################################
# écrire dans le fichier indiqué, ou signaler les erreurs éventuelles
# paramètres :
#   file : le descripteur associé au fichier
#   text : le texte à écrire dans le fichier
def write_file(file, text):
    try:
        file.write(text)  # écrire dans le fichier
    except IOError:  # gestion des erreurs de lecture / écriture
        print("error when writing to the file {}\n".format(
            file.name))  # message d'erreur

##########################################################################################################
# récupérer la valeur de v pour le debug
# paramètres :
#   va : la valeur entière à donner pour activer ou non le mode debug
def get_debug(va):
    # activer (1) ou désactiver (0) le mode debug (affichage d'informations supplémentaires)
    global debug
    debug = va

##########################################################################################################
# enregistrer le chemin d'accès du fichier de configuration
# paramètres :
#   cf : chemin d'accès du fichier de configuration
def get_config_file(cf):
    global config_file
    config_file = cf

##########################################################################################################
# récupérer les paramètres indiqués dans le fichier de configuration
# renvoie :
#   le tuple (port,ws_url,file_ch,log_folder,passive_file,ip,timeout)
def get_param():
    # ouvrir le fichier de configuration ou quitter en cas d'erreur
    try:
        file = open(config_file, 'r')  # ouverture en mode lecture ('r')
    except IOError:
        print("error when opening file" + " " + config_file)
        sys.exit()  # quitter le programme

    # obtenir la taille du fichier de configuration en octets
    t = os.path.getsize(config_file)

    # si le fichier est vide, prévenir l'utilisateur
    if t == 0:
        print("file" + " " + config_file + " " + "is empty please wait\n")
        # fermer le fichier
        file.close()

    # sinon, on lit le fichier
    else:
        # informations du fichier de configuration (définies comme variables globales)
        global port  # port de connexion sur le serveur
        global ws_url  # url à ouvrir ex: http://monserveur.mondomaine.com/nws/ManagementConsoleService.asmx
        global file_ch
        global log_folder
        global passive_file
        global nmc  # adresse du NMC
        global ip  # adresse du serveur
        global timeout  # délai d'attente maximum
        global ipp  # ip et port

        global login  # login pour l'authentification
        global password  # pwd pour l'authentification

        # enregistrer les lignes du fichier dans la variable "lines"
        lines = file.readlines()

        # parcourir les lignes du fichier (itération sur "lines")
        for line in lines:  # chaque ligne est enregistrée dans "line"
            # si ca ne commence pas par "#" (commentaire), récupérer la ligne
            # sinon, sauter la ligne
            line = re.sub('\n+', '', line)
            if len(line) != 0 and line[0] != '#':
                # détecter la présence du signe "=" (expression régulière) et distinguer les membres de part et d'autre
                #print("{} - {}".format(line, len(line)))
                words = re.split(r'\=+', line)

                # indice
                #i = 1

                # supprimer les retours chariot "\n"
                #word = re.sub('\n+', '', words[i])

                # si le premier mot est "port" (terme à gauche de "=" dans le fichier de config), récupérer le mot suivant ...
                # on fait de même pour les autres paramètres
                if words[0] == "port":
                    port = words[1].strip()
                elif words[0] == "url":
                    ws_url = words[1].strip()
                elif words[0] == "newtest_output_folder":
                    file_ch = words[1].strip()
                elif words[0] == "newtest_log_folder":
                    log_folder = words[1].strip()
                elif words[0] == "passive_file":
                    passive_file = words[1].strip()
                elif words[0] == "ip":
                    ip = words[1].strip()
                elif words[0] == "timeout":
                    timeout = words[1].strip()
                elif words[0] == "login":
                    login = words[1].strip()
                elif words[0] == "password":
                    password = words[1].strip()

        nmc = ip + "/NMC"
        ipp = ip + ":" + port

    return(port, ws_url, file_ch, log_folder, passive_file, ip, timeout)

##########################################################################################################
def get_file_rbt_plug():
    return file_r

##########################################################################################################
# enregistrer l'adresse du NMC
def get_nmc(adrr):
    nmc = adrr

##########################################################################################################
def get_road():
    return file_ch

##########################################################################################################
def get_road_log():
    return log_folder

##########################################################################################################

    ###############################################################################
    #######			                                            #######
    #######			        Robot    	                        #######
    #######			                                            #######
    ###############################################################################
#######################################	début robot ######################################################

##########################################################################################################
# requête pour récupérer des informations sur l'état des robots
def request_robot(srv):
    # exécute la requête (de type GET) et enregistre la réponse
    # try:
    #    req = urllib.request.urlopen("http://" + nmc)

    # détection des erreurs potentielles lors de l'envoi ou la réception de la requête
    # except (urllib.error.HTTPError, urllib.error.URLError) as e:
    #    if debug==1: print("exécution de la requête get échouée")

    # else:
    #    rep = req.read().decode("utf8") #réponse reçue (conversion en utf8)
    #    if debug==1: print("requête get exécutée avec succès")

    # création de l'entête SOAP et du message
    message = """<?xml version=\"1.0\" encoding=\"utf-8\"?>
    <soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">
    <soap:Body><ListRobotStatus xmlns=\"http://www.auditec-newtest.com\">
    <mode>All</mode>
    </ListRobotStatus>
    </soap:Body>
    </soap:Envelope>"""

    donnees = message.encode('utf8')  # encodage des données à envoyer

    # entête SOAP
    header = {"Content-Type": "text/xml; charset=utf-8",
              "Content-Length": len(donnees),
              "SOAPAction": "http://www.auditec-newtest.com/ListRobotStatus"}

    # création de la requête en mode "POST" au serveur, dont l'adresse a été introduite par l'utilisateur en ligne de commande
    request = urllib.request.Request(ws_url, donnees, header)
    # ajout des infos d'authentification
    if debug == 1:
        print("Exécution requête POST : " + ws_url)
    try:
        # Ajout de l'authentification
        if (login != ""):
            base64string = base64.b64encode(("%s:%s" % (login, password)).encode())
            request.add_header("Authorization", "Basic %s" % str(base64string, "utf-8"))


        # envoi de la requête au serveur et décodage
        response = urllib.request.urlopen(request).read().decode("utf8")

    # réponse du serveur
    # traitement des erreurs liées à HTTP, et les autres (refus de connexion,...)
    except (urllib.error.HTTPError, urllib.error.URLError) as e:
        write_file(TRACE, "the response of the agent is '{}' with the following error message: {}\n".format(e.code, e.reason))
        if (e.code == 401):
            write_file(TRACE, "Invalid login or password")

        if debug == 1:
            print(response)
        sys.exit(1)  # quitter si la requête get a été un échec

    else:  # si le code de retour est 200 (la requête a abouti)
        # renvoyer la réponse du serveur (sous la forme d'une chaîne XML)
        if debug == 1:
            write_file(TRACE, "the response of the agent is:{}".format(response))

        return response  # chaîne (de caractères) XML

##########################################################################################################
def parser_robot_v2(x):  # deuxième version du parseur de la requête sur les robots
    # parser la chaîne XML "x", et stocker le résultat sous la forme d'un dictionnaire dans "raf"
    raf = xmltodict.parse(x)

    # initialisation des variables globales (liste des robots, et leur nombre total)
    global list_robot
    list_robot = [['', '', '']]

    global cpt_robot
    cpt_robot = 0

    # récupérer les informations depuis "raf"
    for xmlrob in raf['soap:Envelope']['soap:Body']['ListRobotStatusResponse']['ListRobotStatusResult']['RobotItem']:
        list_robot[cpt_robot][0] = xmlrob['Name'].upper()
        list_robot[cpt_robot][1] = xmlrob['Status'].upper()
        list_robot[cpt_robot][2] = xmlrob['LastMessage'].upper()
        cpt_robot += 1  # compter le nombre de robots
        list_robot.append(['', '', ''])  # ajouter une ligne à la liste

##########################################################################################################
def recup_robot():  # récupération du tableau concernant les informations sur les robots (list_robot),
                   # et de leur nombre (cpt_robot)
    # afficher la liste des robots
    if debug == 1:
        print("\n --------------------------------affichage des résultats-------------------------- \n")
        for i in range(cpt_robot):
            for j in range(3):
                print("\n", i, j, list_robot[i][j], "\n")

    return(cpt_robot, list_robot)

##########################################################################################################
def get_file_rbtsce(file):  # récupérer le nom complet du fichier file_rbt_sce
    if debug == 1:
        print("\n ici file=" + file)

    global file_r
    file_r = file + ".txt"

    t = int(floor(time.time()))

    # fichier temporaire qui contiendra les informations sur les robots et leur état, et les scénarios en cours
    # pendant l'exécution de Newtest_pooler.py
    global file_tmp
    file_tmp = "{}_tmp.{}".format(file_r, t)

    global FILE_R

    try:
        FILE_R = open(file_tmp, 'a+')  # création du fichier file_tmp
        # mode 'a': ajout à la suite fichier indiqué
        # mode 'a+' : idem mais si le fichier est inexistant, il est créé automatiquement
    except IOError as e:
        print("impossible d'ouvrir {}".format(file_tmp))
        if debug == 1:
            # code d'erreur renvoyé
            print("code d'erreur : {}".format(e.errno))

##########################################################################################################
def get_file_tmp():
    return file_tmp

##########################################################################################################
def get_desc_trace():
    return TRACE

##########################################################################################################
def get_file_trace(file):  # récupérer le nom complet du fichier trace
    if debug == 1:
        print("\n ici file_trace=" + file)
    file_t = file

    global TRACE

    while os.path.isfile(file_t):  # si le fichier file_r existe déjà...
        os.unlink(file_t)   # ... on supprime ce dernier

    try:
        TRACE = open(file_t, 'a+')
    except IOError:
        print("impossible d'ouvrir le fichier {}".format(file_t))

##########################################################################################################
# écrire les informations sur les robots dans le fichier file_rbt_sce
def writing_robot(b, cpt_rbt, list_rbt):
    list_robot = list_rbt
    cpt_robot = cpt_rbt

    global base
    base = b

    now = time.ctime()  # obtenir la date et l'heure actuelles

    # écrire dans le fichier trace
    write_file(
        TRACE, "***** start : OPEN file_rbt_sce_base at : {} *****\n".format(now))

    # écrire dans file_r l'état des robots (obtenu lors de la première requête dans request_robot)
    for i in range(cpt_robot):
        write_file(FILE_R, word_rbt + "||" +
                   list_robot[i][0] + "||" + list_robot[i][1] + "||" + list_robot[i][2] + "\n")

        if debug == 1:
            print(word_rbt + "||" + list_robot[i][0] + "||" +
                  list_robot[i][1] + "||" + list_robot[i][2] + "\n")
    if debug == 1:
        print("\n remplissage du fichier " + file_r + "\n")


#######################################      fin robot     ###############################################

        ########################################################
        #######                                         ########
        #######		       scénario    	      ########
        #######                                         ########
        ########################################################
#######################################	  début scénario   ###############################################
def req_sce(name):  # requête n°1 sur les scénarios (listscenariostatus)
    if debug == 1:
        print("name=", name)

    # entête et message
    message = """<?xml version=\"1.0\" encoding=\"utf-8\"?>
    <soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">
    <soap:Body>
    <ListScenarioStatus xmlns=\"http://www.auditec-newtest.com\">
    <mode>Robot</mode>
    <range>0</range>
    <args>
    <string>{}</string>
    </args>
    </ListScenarioStatus>
    </soap:Body>
    </soap:Envelope>""".format(name)
    
    # données à envoyer
    donnees = message.encode('utf8')

    header = {"Content-Type": "text/xml; charset=utf-8",
              "Content-Length": len(donnees),
              "SOAPAction": "http://www.auditec-newtest.com/ListScenarioStatus"}

    # création de la requête en mode "POST" au serveur dont l'adresse a été introduite par l'utilisateur en ligne de commande
    request = urllib.request.Request(ws_url, donnees, header)
    if debug == 1:
        print("Exécution requête POST : " + ws_url)

    try:
        if (login != ""):
            base64string = base64.b64encode(("%s:%s" % (login, password)).encode())
            request.add_header("Authorization", "Basic %s" % str(base64string, "utf-8"))

        # envoi de la requête au serveur
        response = urllib.request.urlopen(request).read().decode("utf8")

    # réponse du serveur
    # traitement des erreurs liées à HTTP, et les autres (refus de connexion,...)
    except (urllib.error.HTTPError, urllib.error.URLError) as e:
        print("erreur requete post")
        print("the response of the agent is:", e.code)
        if debug == 1:
            print(response)
        sys.exit()

    else:  # si le code de retour est 200 (la requête a abouti)
        # récupération de la réponse du serveur(texte xml) dans une variable
        y = response
        if debug == 1:
            print("\n recuperation de la requete scenario réussie \n")
            print(y)
            write_file(TRACE, "\nthe result of the second request is:\n " + y + "\nend of the second request\n")

    return y

##########################################################################################################
def get_name(nam):  # enregistrer le nom du robot en cours
    if debug == 1:
        print("\n name=", nam)
    global name
    name = nam

##########################################################################################################
def request_sce(robotName, scenarioName):  # requête n°2 sur les scénarios (listresult)
    # entête et message
    message = """<?xml version=\"1.0\" encoding=\"utf-8\"?>
    <soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">
    <soap:Body>
    <ListResults xmlns=\"http://www.auditec-newtest.com\">
    <mode>Instance</mode>
    <range>30</range>
    <args>
    <string>{}</string>
    <string>{}</string>
    </args>
    </ListResults>
    </soap:Body>
    </soap:Envelope>""".format(scenarioName, robotName)
    donnees = message.encode('utf8')

    header = {"Content-Type": "text/xml; charset=utf-8",
              "Content-Length": len(donnees),
              "SOAPAction": "http://www.auditec-newtest.com/ListResults"}

    # création de la requête en mode "POST" au serveur dont l'adresse a été introduite par l'utilisateur en ligne de commande
    request = urllib.request.Request(ws_url, donnees, header)
    if debug == 1:
        print("Exécution requête POST : " + ws_url)

    try:
        if (login != ""):
            base64string = base64.b64encode(("%s:%s" % (login, password)).encode())
            request.add_header("Authorization", "Basic %s" % str(base64string, "utf-8"))

        # envoi de la requête au serveur
        response = urllib.request.urlopen(request).read().decode("utf8")

    # réponse du serveur
    # traitement des erreurs liées à HTTP, et les autres (refus de connexion,...)
    except (urllib.error.HTTPError, urllib.error.URLError) as e:
        print("erreur requete post")
        print("the response of the agent is:", e.code)
        sys.exit(1) #exit if the request failed

    else:  # si le code de retour est 200
        # recuperation de la reponse du serveur(texte xml) dans une variable
        z = response
        #print("\nthe result of the second request is:\n " + z + "\n")
        if debug == 1:
            print(z)

    return z

##########################################################################################################
# parseur de la requête n°1 sur scénario
def sce_parser_v2(response):
    # parser la chaîne xml "x" et la stocker sous forme de dictionnaire
    # l'argument force_list force l'utilisation de dictionnaires,
    raf = xmltodict.parse(response, force_list={'InstanceScenarioItem'})
    # même si celui-ci ne contient qu'un seul élément
    # enregistrer la liste des statuts des différents scénarios
    R = raf['soap:Envelope']['soap:Body']['ListScenarioStatusResponse']['ListScenarioStatusResult']

    # si cette catégorie existe (en particulier, si le robot n'est pas en panne)
    if R != None:
        # scénarios et leurs attributs
        for xmlsce in R['InstanceScenarioItem']:
            scenarioName = xmlsce['MeasureName'].upper()  # nom du scénario
            scenarioStatus = xmlsce['Status']  # statut d'exécution
            # date du dernier message
            scenarioLastMessage = xmlsce['LastMessage']

            if debug == 1:
                write_file(TRACE, "Write scenario data into temp file\n")

            write_file(FILE_R, "\n{}||{}||{}".format(word_sce, name, scenarioName))  # écrire dans file_tmp

            # requête pour récupérer (entre autres) l'identifiant associé à un scénario donné
            z = request_sce(name, scenarioName)

            if debug == 1:
                write_file(TRACE, "\nthe result of the third request is:\n {}\n".format(z))

            # parser la réponse obtenue
            raf2 = xmltodict.parse(z, force_list={'ResultItem'})

            # extraire les informations sur les scénarios
            R2 = raf2['soap:Envelope']['soap:Body']['ListResultsResponse']['ListResultsResult']

            # s'il y a des scénarios en cours d'exécution sur la machine :
            if R2 != None:
                # extraire l'identifiant du scénario étudié
                resultNode = R2['ResultItem'][0]
                resultId = resultNode['Id']

                # enregistrer les informations (identifiant,statutScenario,dateDernierMessage) dans file_tmp
                write_file(FILE_R, "={}||{}||{}||".format(
                    resultId, scenarioStatus, scenarioLastMessage))

                # nouvelle requête : récupérer les informations de performances pour un scénario donné
                resultChildren = req_resultchildren(resultId)

                # parser la réponse obtenue
                raf3 = xmltodict.parse(
                    resultChildren, force_list={'ResultItem'})

                rCN1 = raf3['soap:Envelope']['soap:Body']['ListResultChildrenResponse']

                # ensemble des résultats de la requête
                rCN2 = rCN1['ListResultChildrenResult']

                # dans le cas où l'on a ces données de performances :
                if rCN2 != None:
                    for resultChildNode in rCN2['ResultItem']:

                        # disjonction de cas selon la taille de l'ensemble des résultats obtenus,
                        # nécessaire à cause de la gestion des chaînes XML par la librairie xmltodict
                        if len(resultChildNode) == 1:
                            # récupérer la valeur contenue dans le dictionnaire
                            rcnode = resultChildNode.values()
                        else:
                            rcnode = resultChildNode  # sinon, récupérer le dictionnaire lui-même

                        # récupérer les valeurs correspondant aux types de mesures (clés 'MeasureName')
                        measureName = rcnode['Measure']['Name']
                        executionValue = rcnode['ExecutionValue']
                        threshold = rcnode['Measure']['Threshold']

                        if threshold == -79228162514264337593543950335:
                            threshold = -1

                        # écrire ces informations dans file_tmp
                        write_file(FILE_R, "{}={};{}!!".format(measureName, executionValue, threshold))
                    # séparateur de fin de ligne
                    write_file(FILE_R, "||")

                else:
                    # séparateur de fin de ligne
                    write_file(FILE_R, "||")

##########################################################################################################

               ##############################################
               ########                             #########
               ########       Mesures filles  	    #########
               ########                             #########
               ##############################################

##########################################################################################################
# récupérer les informations de performances pour un scénario donné
def req_resultchildren(resultId):
    if debug == 1:
        write_file(TRACE, "fourth request - {}\n".format(resultId))

    message = """<?xml version=\"1.0\" encoding=\"utf-8\"?>
    <soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">
    <soap:Body>
    <ListResultChildren xmlns=\"http://www.auditec-newtest.com\">
    <resultId>{}</resultId>
    </ListResultChildren>
    </soap:Body>
    </soap:Envelope>""".format(resultId)

    donnees = message.encode('utf8')

    header = {"Content-Type": "text/xml; charset=utf-8",
              "Content-Length": len(donnees)}

    # création de la requête en mode "POST" au serveur dont l'adresse a été introduite par l'utilisateur en ligne de commande
    request = urllib.request.Request(ws_url, donnees, header)
    if debug == 1:
        print("Exécution de la requête POST : " + ws_url)

    try:
        if (login != ""):
            base64string = base64.b64encode(("%s:%s" % (login, password)).encode())
            request.add_header("Authorization", "Basic %s" % str(base64string, "utf-8"))

        # envoi de la requete au serveur
        response = urllib.request.urlopen(request).read().decode("utf8")

    # réponse du serveur
    # traitement des erreurs liées à HTTP, et les autres (connexion refusée,...)
    except (urllib.error.HTTPError) as e:
        print("erreur requete post")
        print("the response of the agent is:", e.code)

    else:  # si le code de retour est 200
        if debug == 1:
            print("\n resultat de la requete children\n")
            print(response)
            write_file(TRACE, "\nthe result of the fourth request is :\n{}\n".format(response))

        return response

##########################################################################################################
# gérer la fin de l'exécution du pooler
def end():
    now1 = time.ctime()
    now2 = time.ctime()  # pour afficher le moment où le fichier TRACE finit de se remplir

    write_file(
        TRACE, "\n end of the \"file file_rbt_sce_...\" : {}\n***** {} *****\n ".format(now1, now2))

    now4 = time.ctime()

    # pour afficher le moment où l'exécution du pooler est terminée
    print("\n end : {} \n".format(now4))
    FILE_R.close()  # fermer file_tmp

    # on écrase le fichier file_r s'il existe déjà, avec les nouvelles données sur les scénarios,
    # présentes dans le fichier temporaire file_tmp
    while os.path.isfile(file_r):  # si le fichier file_r existe déjà...
        os.unlink(file_r)   # ... on supprime ce dernier

    # écrire dans trace la suppression du chemin d'accès vers l'ancien fichier file_r
    write_file(TRACE, "File exist ... Unlink! .")

    try:  # puis, on (re)crée le fichier file_r, et on copie les données du fichier temporaire file_tmp dedans
        # ouverture du fichier file_r, en mode "ajout" ('a')
        FILE = open(file_r, 'a')
    except IOError:
        print("can't open {}".format(file_r))
    else:
        try:
            # copier le contenu du fichier temporaire (file_tmp) dans le fichier définitif (file_r)
            copyfile(file_tmp, file_r)
        except IOError:
            write_file(TRACE, "File {} cannot be copied.".format(file_tmp))
        else:
            # accorder les droits de lecture / écriture / exécution
            os.chmod(file_r, 0o0777)

    try:
        os.unlink(file_tmp)  # supprimer le fichier temporaire file_tmp
    except IOError:
        print("error : can't unlink {}".format(file_tmp))

    FILE.close()
    TRACE.close()

##########################################################################################################
# renvoyer un 3-uplet d'informations (état nagios, exit nagios, perf) selon le statut du robot
def get_nagios_rbt_state(stat):
    tab = []

    # colonnes : 0 = etat nmc, 1 = etat nagios, 2 = exit nagios, 3 = perf
    tab.append(["OK", "OK", "0", "10"])
    tab.append(["SUSPENDED", "WARNING", "1", "5"])
    tab.append(["STOPPED", "WARNING", "1", "5"])
    tab.append(["UNKNOWN", "CRITICAL", "2", "0"])
    tab.append(["UNINSTALLED", "CRITICAL", "2", "0"])
    tab.append(["INOPERATIVE", "CRITICAL", "2", "0"])
    tab.append(["RESUME", "WARNING", "1", "5"])

    if stat == "OK":
        ret = (tab[0][1], tab[0][2], tab[0][3])
    elif stat == "SUSPENDED":
        ret = (tab[1][1], tab[1][2], tab[1][3])
    elif stat == "STOPPED":
        ret = (tab[2][1], tab[2][2], tab[2][3])
    elif stat == "UNKNOWN":
        ret = (tab[3][1], tab[3][2], tab[3][3])
    elif stat == "UNINSTALLED":
        ret = (tab[4][1], tab[4][2], tab[4][3])
    elif stat == "INOPERATIVE":
        ret = (tab[5][1], tab[5][2], tab[5][3])
    elif stat == "RESUME":
        ret = (tab[6][1], tab[6][2], tab[6][3])

    return ret

##########################################################################################################
# renvoyer un couple d'informations (état nagios, exit nagios) selon le statut du scénario
def get_nagios_sce_state(stat):

    tab = []

    # colonnes : 0 = etat nmc, 1 = etat nagios, 2 = exit nagios
    tab.append(["Available", "OK", "0"])
    tab.append(["Warning", "WARNING", "1"])
    tab.append(["Failed", "CRITICAL", "2"])
    tab.append(["Canceled", "OK", "0"])
    tab.append(["Unknown", "UNKNOWN", "3"])
    tab.append(["OutOfRange", "UNKNOWN", "3"])

    if stat == "Available":
        ret = (tab[0][1], tab[0][2])
    elif stat == "Warning":
        ret = (tab[1][1], tab[1][2])
    elif stat == "Failed":
        ret = (tab[2][1], tab[2][2])
    elif stat == "Canceled":
        ret = (tab[3][1], tab[3][2])
    elif stat == "Unknown":
        ret = (tab[4][1], tab[4][2])
    elif stat == "OutOfRange":
        ret = (tab[5][1], tab[5][2])

    return ret

# enregistrer le nom du plugin correspondant aux informations sur les robots
def get_plug_rbt_name():
    global name_plug_rbt
    name_plug_rbt = "Newtest_plugin_rbt.py"
    return name_plug_rbt

# enregistrer le nom du plugin correspondant aux informations sur les scénarios
def get_plug_sce_name():
    global name_plug_sce
    name_plug_sce = "Newtest_plugin_rbt_sce.py"
    return name_plug_sce

# renvoyer un message d'erreur si le nombre de paramètres fournis au plugin robot est incorrect
def more_param(arg):
    passif = get_passif_arg()
    if len(arg) < 4:
        print("the file needs more parameters \n the parameters are: \n the name of the config file Newtest ex:/tmp/newtest_config.cfg \n the case of execution ex:for the passif case 'p' and for the actif 'a'\nthe adress of the NMC ex:192.168.1.2\nthe name of the robot ex:berlin\nfor the passif case :the name of the host (only for the file concerning the plugin scenario) ex:\n host_berlin  \n ex: perl Newtest_plugin_rbt.pl c:\newtest_config.cfg a 192.168.8.121 berlin ---> actif case\nex: perl Newtest_plugin_rbt.pl c:\newtest_config.cfg p 192.168.8.121 berlin host_berlin --> passif case")
        res = "no-ok"

    elif arg[1] == "passif" and len(arg) == 4:
        print("the file needs an other parameter which is the name of the host \n ex: host_berlin ")
        res = "no-ok"

    else:
        res = "ok"

    return res

# enregistrer le nom du plugin robot
def name_plugin_rbt():
    global plug_rbt
    plug_rbt = "Newtest_plugin_rbt.py"
    return plug_rbt

# enregistrer le nom du plugin scénario
def name_plugin_rbt_sce():
    global plug_rbt_sce
    plug_rbt_sce = "Newtest_plugin_rbt_sce.py"
    return plug_rbt_sce

# obtenir les préfixe et suffixe permettant de construire le chemin d'accès
# vers les données obtenues par le pooler
def get_suffix_preffix_file_pooler():
    suffix_file_pooler = "Newtest_file_rbt_sce_"
    preffix_file_pooler = '.txt'
    return (suffix_file_pooler, preffix_file_pooler)

# obtenir les préfixe et suffixe permettant de construire le chemin d'accès
# vers le log du plugin robot
def get_suffix_preffix_trace_plug_rbt():
    suffix_trace_plug_rbt = "trace_Newtest_plugin_rbt_"
    preffix_trace_plug_rbt = '.log'
    return (suffix_trace_plug_rbt, preffix_trace_plug_rbt)

# obtenir les préfixe et suffixe permettant de construire le chemin d'accès
# vers le log du plugin scénario


def get_suffix_preffix_trace_plug_sce():
    suffix_trace_plug_sce = "trace_Newtest_plugin_rbt_sce_"
    preffix_trace_plug_sce = '.log'
    return (suffix_trace_plug_sce, preffix_trace_plug_sce)

# obtenir le caractère définissant le mode passif
def get_passif_arg():
    passif_arg = "p"
    return passif_arg

# obtenir le mot clé associé à un robot
def get_word_for_robot():
    return word_rbt

# obtenir le mot clé associé à un scénario
def get_word_for_scenario():
    return word_sce

# obtenir les états possibles pour un robot
def get_rbt_state():
    tab = ["OK", "SUSPENDED", "STOPPED", "UNKNOWN", "UNINSTALLED", "INOPERATIVE", "RESUME"]
    return tab

# renvoyer le chemin d'accès vers le plugin 'Newtest_plugin_rbt_sce"
def exec_plug_sce_passif():
    name_plug_rbt = get_plug_rbt_name()
    name_plug_rbt_sce = get_plug_sce_name()

    # chemin d'accès absolu vers le plugin
    road_sce = os.path.dirname(os.path.realpath(__file__)) + "/" + name_plug_rbt_sce
    return road_sce

# renvoyer un message d'erreur si le nombre de paramètres fournis au plugin scénario est incorrect
def more_param_sce(arg):
    passif = get_passif_arg()
    if len(arg) < 5:
        print("the file needs more parameters\nthe parametrs are:\nthe name of the config file Newtest ex:/tmp/newtest_config.cfg\nthe case of execution ex:for the passif case 'p' and for the actif case 'a'\nthe adress of the NMC ex:192.168.1.2\nthe name of the robot ex:berlin\nthe name of the host for the passif case or the name of the scenario the actif case(only for the file concerning the plugin scenario) ex:\n host_berlin (passif case) or dns_test(actif case for the file concerning the plugin scenario)\nex: perl Newtest_plugin_rbt_sce.py c:\\newtest_config.cfg a 192.168.8.121 berlin dns_test ---> actif case\nex: perl Newtest_plugin_rbt_sce.py c:\\newtest_config.cfg p 192.168.8.121 berlin host_berlin ---> passif case")
        return "no-ok"