#!/usr/bin/python3
# -*- coding: utf-8 -*-

####################################################################################
# Projet : traduction du pooler Newtest, du langage Perl vers Python,              #
#        ce script utilise la librairie Newtestlib.py pour enregistrer les données #
#        collectées depuis les machines Newtest dans un fichier texte              #
#                                                                                  #
# Auteur : Sherwin SABRAS (https://www.linkedin.com/in/sherwin-sabras-321164172/)  #
# Date de création : 3 juin 2019                                                   #
#                                                                                  #
# Release : 2019-09-24 YSI  v2.0.1                                                 #
####################################################################################

import sys
import os
import re
import datetime
import Newtestlib as Ntl

########################################################################################
#                                                                                       #
#                               file POOLER Version 2 - PYTHON                          #
#                                                                                       #
#########################################################################################

##########################################################################################################

##########################################################################################################
# liste contenant les arguments de la ligne de commande
# argument 0 : chemin d'accès du fichier de configuration
# argument 1 : mode debug
arg = sys.argv

# affichage d'un message d'erreur s'il manque des arguments
if len(arg) < 2:
    print("missing argument argv0 is the config file path and the argv1 is the debug flag 0 or 1\n")
    sys.exit()

path = os.path.basename(__file__)

# on supprime le premier argument entré via la ligne de commande
# (correspondant au chemin d'accès du script exécuté)
# if arg[0] == path:
if os.path.basename(arg[0]) == path:
    del arg[0]

# s'il y a trop d'arguments
if len(arg) > 2:
    print("too many arguments : argv0 is the config file path and the argv1 is the debug flag 0 or 1\n")
    sys.exit(1)

# mode debug activé
if int(arg[1]) == 1:
    print("v=1\n")
    debug = 1
else:
    debug = 0

# quitter si le fichier de configuration n'existe pas
if not os.path.isfile(arg[0]):
    print("config file {} doesn't exist \n".format(arg[0]))
    sys.exit(1)

Ntl.get_config_file(arg[0])  # recuperer le chemin du fichier de configuration
# recuperer les parametres à partir du fichier /tmp/newtest_config.cfg
ba = Ntl.get_param()

base = ba[5]  # récupère l'adresse ip

if len(ba) > 6:
    timeout = int(ba[6])  # recuperer le timeout

if debug == 1:
    print("base: {}\n".format(base))

# pour que le nom de fichier ne contienne pas de '.' mais des '_' car Linux n'accepte pas les '.'
f = base.replace(".", "_")

if debug == 1:
    print("base: {}\n".format(f))

srv = "http://" + base  # car il faut qu'elle commence par 'http://'

# exécuter get_road pour recuperer le chemin où seront mis les fichier file_rbt_sce...
trace = "{}Newtest_trace_pooler_{}.txt".format(Ntl.get_road_log(), f)

# ...et donner le nom du fichier trace a la librairie
Ntl.get_file_trace(trace)
TRACE = Ntl.get_desc_trace()

# recuperation des états des robots
Ntl.write_file(TRACE, "Start request for Newtest robots\n")
response_robot = Ntl.request_robot(srv)

if debug == 1:
    print("\n{}\n".format(response_robot))
    Ntl.write_file(TRACE, "\n **** FILE TRACE *****\n\n the response of the request:\n    {} \n end of the first request to the server(robot)\n".format(response_robot))

# parser la chaîne XML résultat de la requête sur les robots
Ntl.parser_robot_v2(response_robot)
(cpt_robot, list_robot) = Ntl.recup_robot()

Ntl.write_file(TRACE, "Number of robots found : {}\n".format(cpt_robot))

# affichage des résultats de la requête
if debug == 1:
    print("\n -------------------------------- results -------------------------- \n")
    for i in range(cpt_robot):
        for j in range(3):
            print("{}{}  {}".format(i, j, list_robot[i][j]))

############### création et remplissage du fichier de robot_scenario file_rbt_sce_nom serveur ############################
if debug == 1:
    Ntl.write_file(TRACE, "Writing of the name,status and lastmessage of all robots in the file file_rbt_sce_{}\n".format(base))

file_r = "{}Newtest_file_rbt_sce_{}".format(Ntl.get_road(), f)
Ntl.get_file_rbtsce(file_r)  # donner le nom du fichier à la librairie
if debug == 1:
    print("\nbase={}".format(base))
# remplissage des informations sur les robots dans un tableau list_robot
Ntl.writing_robot(base, cpt_robot, list_robot)

if debug == 1:
    Ntl.write_file(TRACE, "end of Writing of the name,status and lastmessage of all robots in the file file_rbt_sce_{}\n".format(base))

####pour chaque robot récupérer tous ces scénarios ainsi que leur état et leur last message####
if debug == 1:
    Ntl.write_file(TRACE, "second request to the server(for each robot get the name,status and last message of his scenarios)\nboucle for\n")

# start timeout for the loop
currenttime = datetime.datetime.now()  # date et heure actuelles
# heure de fin maximale (on additionne currenttime et timeout (en secondes))
endtime = currenttime + datetime.timedelta(seconds=timeout)
Ntl.get_debug(debug)

#Ntl.write_file(TRACE, "cpt_robot = {}".format(cpt_robot))

cpt = 0
sortie = False

# boucle sur tous les robots (nom de robot)
while cpt <= cpt_robot and not sortie:
    if list_robot[cpt][0] == '':
        Ntl.write_file(TRACE, "In notdefined with cpt= {}\n".format(cpt))
        Ntl.end()  # sortir de la librairie
        sortie = True
    else:
        name = list_robot[cpt][0]
        Ntl.write_file(TRACE, "Get scenario data for = {}\n".format(name))

        # récupérer le nom du robot en cours pour l'utiliser dans la librairie
        Ntl.get_name(name)
        for j in range(len(Ntl.liste_sce)):
            # remettre tous les booléens a 0 ca va etre un autre robot
            Ntl.liste_sce[j][4] = 0

        if debug == 1:
            print("robot name :  {} \n".format(name))
        # requête pour recuperer des informations sur les scenarios (dans un fichier xml)
        response_sce = Ntl.req_sce(name)
        if debug == 1:
            print("y=" + response_sce)
            Ntl.write_file(TRACE, "parser xml for the xml result of the second request to the server(scenarios)\n")

        # parser le resultat du fichier xml scenario puis continuer l'ensemble du travail
        Ntl.sce_parser_v2(response_sce)

        if datetime.datetime.now() > endtime:
            # si le timeout est dépassé, quitter
            Ntl.write_file(TRACE, "timeout")
            Ntl.end()  # sortir de la librairie
            sortie = True
    cpt += 1